/*
 * Creator: Naman Dixit
 * Notice: © Copyright 2020 Naman Dixit
 */

/*
 * Creator: Naman Dixit
 * Notice: © Copyright 2019 Naman Dixit
 */

#include "nlib/nlib.h"

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <netdb.h>
#include <unistd.h>
#include <ctype.h>

# if defined(COMPILER_CLANG)
#  pragma clang diagnostic push
#   pragma clang diagnostic ignored "-Wpadded"
#   pragma clang diagnostic ignored "-Wfloat-equal"
# endif
#include "cJSON/cJSON.h"
#include "cJSON/cJSON.c"
# if defined(COMPILER_CLANG)
#  pragma clang diagnostic pop
# endif


int main(int argc, char** argv)
{
    if (argc < 2) {
        printf("Error: Pass the amount of memory\n");
        exit(-1);
    }

    Sint memory_required = (Sint)strtol(argv[1], NULL, 10);

    Char *port = "9526";

    Sint sock_fd = socket(AF_INET, SOCK_STREAM, 0);

    struct addrinfo hints = {.ai_family = AF_INET,
                             .ai_socktype = SOCK_STREAM};
    struct addrinfo *result = NULL;
    Sint s = getaddrinfo(NULL, port, &hints, &result);
    if (s != 0) {
        fprintf(stderr, "Error: getaddrinfo: %s\n", gai_strerror(s));
        exit(-1);
    }

    while (connect(sock_fd, result->ai_addr, result->ai_addrlen) == -1) {
        fprintf(stderr, "Error: Couldn't connect on port %s, trying again in one second...\n", port);
        sleep(1);
    }

    printf("Log: Starting communication with server on port %s...\n", port);

    Char *output = NULL;

    sbufPrint(output, "    ");
    sbufPrint(output, "{\n\"id\": %d", (Sint)time(NULL));
    sbufPrint(output, ",\n\"memory\": %d", memory_required);
    sbufPrint(output, "\n}\n");
    Size output_len = strlen(output);

    printf("Sending to Arbiter:\n%s\n",
           cJSON_Print(cJSON_Parse(output + 4)));

#if defined(ENDIAN_LITTLE)
    U32 json_len = (U32)output_len - 4;
    U32 json_len_be = swap_endian(json_len);
    output[0] = ((Char*)&json_len_be)[0];
    output[1] = ((Char*)&json_len_be)[1];
    output[2] = ((Char*)&json_len_be)[2];
    output[3] = ((Char*)&json_len_be)[3];
#endif

    write(sock_fd, output, output_len);

    {
        cJSON *array = NULL;

        B32 initialized = false;
        Size buffer_len = 0;
        Size buffer_cap = MiB(1);
        U32 buffer_expected_len = 0;
        Char *buffer = calloc(buffer_cap, sizeof(*buffer));

        Char size_bytes[4] = {0};
        Size size_bytes_count = 0;

        while (true) {
            if (initialized == false) {
                long len = read(sock_fd,
                                (Char*)size_bytes + size_bytes_count,
                                4 - size_bytes_count);

                if (len == 0) {
                    perror("read() returned zero");
                    exit(-1);
                }

                size_bytes_count += (Size)len;

                if (size_bytes_count == 4) {
                    initialized = true;

                    buffer_expected_len = (U32)((size_bytes[3] <<  0U) |
                                                (size_bytes[2] <<  8U) |
                                                (size_bytes[1] << 16U) |
                                                (size_bytes[0] << 24U));
                }

                continue;
            } else {
                long len = read(sock_fd,
                                buffer + buffer_len,
                                buffer_expected_len - buffer_len);

                buffer_len += (Size)len;

                if (buffer_expected_len == buffer_len) {
                    printf("Recieved: Final Response:\n%s\n",
                           cJSON_Print(cJSON_Parse(buffer)));

                    const Char *json_error = NULL;
                    cJSON *root = cJSON_ParseWithOpts(buffer, &json_error, true);

                    if (root == NULL) {
                        // TODO(naman): Error
                    } else {
                        array = cJSON_GetObjectItem(root, "ip");
                    }

                    free(buffer);
                    break;
                }
            }
        }
    }

    return 0;
}
